#include <KD/kd.h>
#include "bubbleWindow.h"
#include "nvgl2demo_common.h"

static BubbleWindow bubbleWindow;
static GLboolean shutdown = GL_FALSE;

// Callback to close window
static void
closeCB(void)
{
    shutdown = GL_TRUE;
}

// Callback to resize window
static void
resizeCB(int width, int height)
{
    BubbleWindow_reshapeViewport(&bubbleWindow, width, height);
}

// Callback to handle key presses
static void
keyCB(char key, int state)
{
    // Ignoring releases
    if (!state) return;

    BubbleWindow_callback(&bubbleWindow, key);
}

#ifndef USE_FAKE_MOUSE
// Callback to move mouse
static void
pointerCB(int x, int y)
{
    BubbleWindow_mouse(&bubbleWindow, x, y);
}

// Callback to handle button presses
static void
buttonCB(int button, int state)
{
    if (button != 1) return; // Only use left mouse
    BubbleWindow_leftMouse(&bubbleWindow, (bool)state);
}
#endif

/* main program */
KDint
kdMain(KDint argc, const KDchar * const *argv)
{
    float duration = 0.0f;
    int autopoke = 0;
    int fpsFlag = 0;

    // Initialize window system and EGL
    if (!NvGl2DemoInitialize(&argc, (char**)argv, "bubble",
                             EGL_OPENGL_ES2_BIT)) {
        return -1;
    }
    // Set up callbacks
    NvGl2DemoSetCloseCB(closeCB);
    NvGl2DemoSetResizeCB(resizeCB);
    NvGl2DemoSetKeyCB(keyCB);
#ifndef USE_FAKE_MOUSE
    NvGl2DemoSetPointerCB(pointerCB);
    NvGl2DemoSetButtonCB(buttonCB);
#endif

    // parse command line options
    argc--;
    argv++;
    while (argc > 0) {
        if (!kdStrcmp(argv[0], "-sec")) {
            if (argc >= 2) {
                duration = (float) kdStrtof(argv[1], KD_NULL);;
                argc -= 2;
                argv += 2;
            } else {
                //NvGl2DemoLogMessage("Insufficient parameters to <%s>\n", argv[0]);
                goto done;
            }
        } else if (!kdStrcmp(argv[0], "-autopoke")) {
            autopoke = kdStrtol(argv[1], KD_NULL, 10);
            argc -= 2;
            argv += 2;
        } else if (!kdStrcmp(argv[0], "-fps")) {
            fpsFlag = 1;
            argc -= 1;
            argv += 1;
        } else {
            //NvGl2DemoLogMessage("Unknown command line option <%s>\n", argv[0]);
            //goto done;
            argc -= 1;
            argv += 1;
			continue;
        }
    }

    NewBubbleWindow(&bubbleWindow, autopoke, duration, fpsFlag);
    BubbleWindow_init(&bubbleWindow, gfxState.width, gfxState.height);

    do {
        // Rendering.
        BubbleWindow_tick(&bubbleWindow);
        eglSwapBuffers(gfxState.display, gfxState.surface);
        NvGl2DemoCheckEvents();
    } while (!bubbleWindow.one_frame && !shutdown);

    if (duration != 0.0f) {
        //NvGl2DemoLogMessage("Exiting after %d seconds\n", (int)duration);

    }

    // Clean up the bubble window
    DestroyBubbleWindow(&bubbleWindow);

    done:
    // Clean up EGL
    NvGl2DemoShutdown();

    return 0;
}
